/*
 * Some functions common to many PIMPPA programs
 *
 */

#include <ctype.h>
#include <fcntl.h>
#include <limits.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libgen.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <utime.h>

#include "pimppa.h"

/*
 * Checks if a string has space in it (" ")
 *
 */
int p_checkforspace(char *string)
{
	int i=0;

	if(!string)
		return(0);

	while(string[i]!=0)
	{
		if(string[i]==' ')
			return(1);
		i++;
	}

	return(0);
}

/*
 * Checks if a string has a number in it (0-9)
 *
 */
int p_checkfornumber(char *string)
{
	int i=0;

	if(!string)
		return(0);

	while(string[i]!=0)
	{
		if(isdigit(string[i]))
			return(1);
		i++;
	}

	return(0);
}

/*
 * Checks if a string has a letter in it (A-Z, a-z)
 * before the last dot.
 *
 */
int p_checkforletter(char *string)
{
	int i=0;

	if(!string)
		return(0);

	while(string[i]!=0)				// Go to the end of string
		i++;

	while(i>=0 && string[i]!='.')	// Rewind until a dot or falloff
		i--;
	
	while(i>=0)						// Found a dot
	{
		if(isalpha(string[i]))		// Found a letter
			return(1);
		i--;
	}

	return(0);
}

/*
 * Creates a "match pattern" from a filename. 
 *
 * All numbers in filename will be replaced with "0".
 * The letters after the last encountered number and before
 * the last dot (".") are thought as indexing, and 
 * replaced with "1". 
 *
 * e.g. "ab-013-f.jpg" => "ab-000-1.jpg"
 *      "ab-013-x.jpg" => "ab-000-1.jpg"
 *      "ab-ccc-1.jpg" => "ab-ccc-0.jpg"
 *      "ab-1-def.jpg" => "ab-0-111.jpg"
 *      "ab-1b-22.jpg" => "ab-0b-00.jpg"
 *
 * This is of course not a foolproof method, but then, what is? =)
 *
 * Output will be written to "output". Returns the
 * number of characters before the first dot.
 *
 */
int p_makepattern(char *output, char *filename)
{
	int i=0, dotfound=0;

	if(!filename || !output)
		return(0);

	while(filename[i]!=0)			// First pass forward, numbers => 0
	{
		if(isdigit(filename[i]))
			output[i]='0';
		else
			output[i]=filename[i];

		if(filename[i]=='.')
			dotfound=i;

		i++;
	}
	output[i]=0;
	
	i=dotfound-1;					// Location of last dot
	
	while(i>0)						// Second pass backwards, letters => 1
	{
		if(isdigit(output[i]))
			break;

		if(isalpha(output[i]))
			output[i]='1';
		
		i--;
	}
	
	return(dotfound);
}

/*
 * Converts a string to lowercase
 *
 */
int p_strtolc(char *string)
{
	int i=0;

	if(!string)
		return(0);

	while(string[i]!=0)
	{
		string[i]=tolower(string[i]);
		i++;
	}

	return(1);
}

/*
 * Connect to the PIMPPA database.
 *
 */
int p_connect(MYSQL *db)
{
	mysql_connect(db, NULL, P_USER, NULL);
	if(mysql_error(db)[0])
	{
		fprintf(stderr, "%s\n", mysql_error(db));
		return(0);
	}
	mysql_select_db(db, P_DBASE);
	if(mysql_error(db)[0])
	{
		fprintf(stderr, "%s\n", mysql_error(db));
		mysql_close(db);
		return(0);
	}
	
	return(1);
}

/*
 * Internal "cp" command to copy files from directory to another.
 * Probably inefficient as hell.
 *
 */
int p_cp(char *src, char *dest)
{
	char buffer[P_CP_BUFFSIZE];
	char truedest[PATH_MAX];
	struct utimbuf origtimes;
	struct stat st;
	int oldfd, newfd, cnt;

	if(!src || !dest)
	{
		fprintf(stderr, "p_cp(): src or dest is null\n");
		return(0);
	}

	if(!src[0] || !dest[0])
	{
		fprintf(stderr, "p_cp(): src[0] or dest[0] is null!\n");
		return(0);
	}
	
	oldfd=open(src, O_RDONLY);
	if(oldfd==-1)
	{
		fprintf(stderr, "p_cp(): Unable to open %s\n", src);
		return(0);
	}
	
	if(fstat(oldfd, &st)!=0)
	{
		fprintf(stderr, "p_cp(): Unable to stat oldfd!\n");
		close(oldfd);
		return(0);
	}

	origtimes.actime=st.st_atime;
	origtimes.modtime=st.st_mtime;

	// Check if destination is a directory
	if(stat(dest, &st)==0 && S_ISDIR(st.st_mode))
	{
//		printf("Basename: %s\n", p_basename(src));
	
		if(!p_checkp(dest))
			sprintf(truedest, "%s/%s", dest, p_basename(src));
		else
			sprintf(truedest, "%s%s", dest, p_basename(src));
	}
	else
		strcpy(truedest, dest);

//	printf("newdest: %s\n", truedest);

	newfd=open(truedest, O_WRONLY|O_CREAT,0644);
	if(newfd==-1)
	{
		fprintf(stderr, "p_mv(): Unable to open %s\n", truedest);
		close(oldfd);
		return(0);
	}

	while((cnt=read(oldfd, buffer, P_CP_BUFFSIZE)))
	{
		write(newfd, buffer, cnt);
	}
	
	close(newfd);
	utime(truedest, &origtimes);	// Restore times
	
	close(oldfd);

	return(1);
}

/*
 * Internal "mv" command. Relies on the "p_cp". 
 *
 */
int p_mv(char *src, char *dest)
{
	if(rename(src,dest)==-1)		// Try this first
	{
		if(!p_cp(src, dest))
			return(0);
	
		unlink(src);
	}

	return(1);
}

/*
 * Returns the filename component of a directory
 *
 */
char *p_basename(char *src)
{
	int i;

	if(!src || !src[0])
		return(NULL);

	for(i=strlen(src);i>=0;i--)
	{
		if(src[i]=='/')
			return(&src[i+1]);
	}

	return(NULL);
}

/* 
 * Returns the first component in "path" in "head"
 * and the rest in "tail". 
 *
 */
int p_pathcomp(char *path, char *head, char *tail)
{
	int i=0;

	if(!path || !path[0])
		return(0);

	if(!head || !tail)	
		return(0);
		
	while(path[i]!=':')
	{
		head[i]=path[i];
		i++;
	}
	head[i]=0;

	strcpy(tail, &path[i+1]);

	return(1);
}

/*
 * Performs a database query. Returns 0 on failure, 1 on success.
 *
 */
int p_query(MYSQL *db, const char * format,...)
{
	va_list args;
	char query[QUERY_MAX];

	va_start(args, format);
	vsnprintf(query, QUERY_MAX, format, args);
	va_end(args);

//	fprintf(stderr, "DEBUG: %s\n", query);

	mysql_query(db, query);
	if(mysql_error(db)[0])
	{
		fprintf(stderr, "SQL error: %s\n", mysql_error(db));
		return(0);
	}
	else
		return(1);
}

/*
 * Return 1 if *path ends in '/', 0 otherwise
 *
 */
int p_checkp(char *path)
{
	if(!path)
		return(0);

	if(path[strlen(path)-1]=='/')
		return(1);
	else
		return(0);
}

/*
 * Replacement for system(). Goes through $PATH.
 *
 */
/*
int p_exec(char *cmd, char *args[])
{
	pid_t mypid;

	mypid=fork();
	if(mypid==0)
	{
		char head[10*PATH_MAX];
		char tail[10*PATH_MAX];
		char *ptr=getenv("PATH");

		p_pathcomp(ptr, head, tail);
		sprintf(args[0], "%s/%s", head, cmd);

		while(execv(args[0], args)==-1)
		{
			if(!p_pathcomp(tail, head, tail))
				break;
				
			sprintf(args[0], "%s/%s", head, cmd);
		}
		exit(-1);
	}
	return(mypid);
}
*/
